@@ -8,18 +8,18 @@ class AnalysisObject(object):
                  fix_param:str='', profile_param:str='', ws_name:Optional[str]=None, 
                  mc_name:Optional[str]=None, snapshot_name:Optional[str]=None,
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', precision:float=0.001, 
-                 eps:float=1.0, strategy:int=1, print_level:int=-1, timer:bool=True,
+                 eps:float=1.0, strategy:int=1, print_level:int=-1, timer:bool=False,
                  num_cpu:int=1, offset:bool=True, optimize:int=2, eigen:bool=False,
                  fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
                  max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
-                 int_bin_precision:float=-1., **kwargs):
+                 int_bin_precision:float=-1., verbosity:Union[int, str]="INFO", **kwargs):
         
         self.model = None
         self.minimizer = None
         
         if filename is not None:
             self.setup_model(filename, data_name, binned_likelihood, ws_name, mc_name, 
-                             snapshot_name, fix_cache, fix_multi)
+                             snapshot_name, fix_cache, fix_multi, verbosity=verbosity)
             self.setup_parameters(fix_param, profile_param)
             self.setup_minimizer(minimizer_type, minimizer_algo, precision, eps, strategy, num_cpu,
                                  offset, optimize, eigen, max_calls, max_iters, print_level,
@@ -27,11 +27,13 @@ class AnalysisObject(object):
         
     def setup_model(self, filename, data_name='combData', binned_likelihood:bool=True, 
                     ws_name:Optional[str]=None, mc_name:Optional[str]=None, 
-                    snapshot_name:Optional[str]=None, fix_cache:bool=True, fix_multi:bool=True):
+                    snapshot_name:Optional[str]=None, fix_cache:bool=True, 
+                    fix_multi:bool=True, verbosity:Union[int, str]="INFO"):
         
         model = ExtendedModel(fname=filename, ws_name=ws_name, mc_name=mc_name, data_name=data_name, 
                               snapshot_name=snapshot_name, binned_likelihood=binned_likelihood,
-                              fix_cache=fix_cache, fix_multi=fix_multi)
+                              fix_cache=fix_cache, fix_multi=fix_multi,
+                              verbosity=verbosity)
         self.model = model
     
     def setup_parameters(self, fix_param:str='', profile_param:str=''):
@@ -45,7 +47,7 @@ class AnalysisObject(object):
     def setup_minimizer(self, minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
                         precision:float=0.001, eps:float=1.0, strategy:int=0, 
                         num_cpu:int=1, offset:bool=True, optimize:bool=True, eigen:bool=False,
-                        max_calls:int=-1, max_iters:int=-1, print_level:int=-1, timer:bool=True,
+                        max_calls:int=-1, max_iters:int=-1, print_level:int=-1, timer:bool=False,
                         constrain_nuis:bool=True, batch_mode:bool=False, int_bin_precision:float=-1.):
         
         minimizer = ExtendedMinimizer("minimizer", self.model.pdf, self.model.data)
@@ -60,7 +62,7 @@ class AnalysisObject(object):
         if hasattr(ROOT.RooFit, "BatchMode"):
             nll_commands.append(ROOT.RooFit.BatchMode(batch_mode))
         if hasattr(ROOT.RooFit, "IntegrateBins"):
-            nll_commands.append(ROOT.RooFit.IntegrateBins(int_bin_precision))          
+            nll_commands.append(ROOT.RooFit.IntegrateBins(int_bin_precision))
         minimizer_options = {
             'minimizer_type'   : minimizer_type,
             'minimizer_algo'   : minimizer_algo,
@@ -79,4 +81,4 @@ class AnalysisObject(object):
         
         self.minimizer = minimizer
         self.minimizer_options = minimizer_options
-        self.nll_commands     = nll_commands        
\ No newline at end of file
+        self.nll_commands      = nll_commands
\ No newline at end of file
