@@ -8,7 +8,7 @@ import sys
 import math
 import logging
 import fnmatch
-from typing import List, Optional, Union, Dict
+from typing import List, Optional, Union, Dict, Set
 
 import numpy as np
 
@@ -468,14 +468,35 @@ class ExtendedModel(object):
         source = self.get_all_constraints()
         return self._unfold_constraints(source, self.observables, self.nuisance_parameters, threshold=threshold)
     
+    @staticmethod
+    def unfold_components(target, source:Set[ROOT.RooAbsArg]):
+        result = ROOT.RooArgSet()
+        if target in source:
+            result.add(target)
+        components = None
+        if isinstance(target, ROOT.RooAbsPdf):
+            components = target.getComponents()
+            components.remove(target)
+        elif isinstance(target, ROOT.RooProduct):
+            components = target.components()
+        if (components is not None) and components.getSize():
+            for component in components:
+                unfolded_components = ExtendedModel.unfold_components(component, source)
+                result.add(unfolded_components)
+        return result
+    
     @staticmethod
     def _pair_constraints(constraint_set:ROOT.RooArgSet, globs:ROOT.RooArgSet, nuis:ROOT.RooArgSet):
         nuis_list = []
         glob_list = []
         pdf_list = []
+        from pdb import set_trace
+        nuis_set = set(nuis)
         for pdf in constraint_set:
             target_np   = None
             target_glob = None
+            # old method (not getting correct nuis)
+            """
             components = pdf.getComponents()
             components.remove(pdf)
             if components.getSize():
@@ -494,15 +515,27 @@ class ExtendedModel(object):
                     if pdf.dependsOn(np):
                         target_np = np
                         break
+            """
+            unfolded_nuis = ExtendedModel.unfold_components(pdf, set(nuis))
+            if not unfolded_nuis:
+                for np in nuis:
+                    if pdf.dependsOn(np):
+                        target_np = np
+                        break
+            elif len(unfolded_nuis) > 1:
+                raise RuntimeError("failed to isolate proper nuisance parameter")
+            else:
+                target_np = unfolded_nuis.first()
+            
             if not target_np:
-                _PRINT_.warning('WARNING: Could not find nuisance parameter for the constrain: {}'.format(pdf.GetName()))
+                _PRINT_.warning('WARNING: Could not find nuisance parameter for the constraint: {}'.format(pdf.GetName()))
                 continue
             for glob in globs:
                 if pdf.dependsOn(glob):
                     target_glob = glob
                     break
             if not target_glob:
-                _PRINT_.warning('WARNING: Could not find global observable for the constrain: {}'.format(pdf.GetName()))
+                _PRINT_.warning('WARNING: Could not find global observable for the constraint: {}'.format(pdf.GetName()))
                 continue
             nuis_list.append(target_np)
             glob_list.append(target_glob)
@@ -532,7 +565,21 @@ class ExtendedModel(object):
     def get_unconstrained_nuisance_parameters(self):
         constrained_nuis = self.get_constrained_nuisance_parameters()
         unconstrained_nuis = list(set(self.nuisance_parameters) - set(constrained_nuis))
-        return unconstrained_nuis    
+        return unconstrained_nuis
+    
+    def set_constrained_nuisance_parameters_to_nominal(self, set_constant:bool=False):
+        constrained_nuis, _, constrain_pdf = self.pair_constraints()
+        for nuis, pdf in zip(constrained_nuis, constrain_pdf):
+            pdf_class = pdf.ClassName()
+            if pdf_class in ["RooGaussian", "RooBifurGauss"]:
+                nuis.setVal(0)
+            elif pdf_class == "RooPoisson":
+                nuis.setVal(1)
+            else:
+                raise RuntimeError(f"constraint term `{pdf.GetName()}` has unsupported type `{pdf_class}`")
+        if set_constant:
+            for nuis in constrained_nuis:
+                nuis.setConstant(1)
     
     def get_all_constraints(self):
         all_constraints = ROOT.RooArgSet()
@@ -620,12 +667,16 @@ class ExtendedModel(object):
                 nuis.removeRange()    
         return None
     
-    def get_poi(self, poi_name:str, strict:bool=False):
-        poi = self.workspace.var(poi_name)
+    def get_poi(self, poi_name:Optional[str]=None, strict:bool=False):
+        if poi_name is None:
+            poi = self.pois.first()
+            _PRINT_.info(f"INFO: POI name not specified. The first POI `{poi.GetName()}` is used by default.")
+        else:
+            poi = self.workspace.var(poi_name)
         if not poi:
-            raise RuntimeError('workspace does not contain the variable "{}"'.format(poi_name))
+            raise RuntimeError(f"workspace does not contain the variable `{poi_name}`")
         if strict and (poi not in list(self.pois)):
-            raise RuntimeError('workspace variable "{}" is not part of the POIs'.format(poi_name))
+            raise RuntimeError(f"workspace variable `{poi_name}` is not part of the POIs")
         return poi
     
     def _load_obs_and_weight(self, obs_and_weight:Optional[Union["ROOT.RooArgSet",str]]=None, 
@@ -691,7 +742,7 @@ class ExtendedModel(object):
         for c, d in pdf_dict.items():
             pdfmap.keepalive.append(d)
             pdfmap.insert(pdfmap.begin(), ROOT.std.pair("const string, RooAbsPdf*")(c, d))
-        return pdfmap    
+        return pdfmap
     
     def generate_asimov_from_pdf(self, name:str="asimovData", pdf:Optional["ROOT.RooAbsPdf"]=None,
                                  obs_and_weight:Optional[Union["ROOT.RooArgSet",str]]=None, 
@@ -738,6 +789,18 @@ class ExtendedModel(object):
         if (asimov_data.sumEntries() != asimov_data.sumEntries()):
             raise RuntimeError("asimov data sum entries is nan")
         return asimov_data
+    
+    def match_globs(self):
+        nuis_list, globs_list, pdf_list = self.pair_constraints()
+        for nuis, globs, pdf in zip(nuis_list, globs_list, pdf_list):
+            pdf_class = pdf.ClassName()
+            if pdf_class in ["RooGaussian", "RooBifurGauss"]:
+                globs.setVal(nuis.getVal())
+            elif pdf_class == "RooPoisson":
+                globs.setVal(nuis.getVal()*globs.getVal())
+            else:
+                raise RuntimeError(f"constraint term `{pdf.GetName()}` has unsupported type `{pdf_class}`")
+            _PRINT_.debug("set glob {} to val {}".format(globs.GetName(), globs.getVal()))
 
     def generate_asimov(self, poi_name:str, poi_val:Optional[float]=None, 
                         poi_profile:Optional[float]=None,
@@ -745,6 +808,7 @@ class ExtendedModel(object):
                         modify_globs:bool=True,
                         do_import:bool=True,
                         asimov_name:Optional[str]=None,
+                        asimov_snapshot:Optional[str]=None,
                         channel_asimov_name:Optional[str]=None,
                         dataset:Optional["ROOT.RooDataSet"]=None,
                         constraint_option:int=0,
@@ -791,6 +855,9 @@ class ExtendedModel(object):
                     Name of the generated asimov dataset. If None, defaults to "asimovData_{mu}" where
                     `{mu}` will be replaced by the value of `poi_val`. Other keywords are: `{mu_cond}`
                     which is the the value of `poi_profile`.
+                asimov_snapshot: (Optional) str
+                    Name of the snapshot taken right after asimov data generation. If None, no snapshot
+                    will be saved.
                 channel_asimov_name: (Optional) str
                     Name of the asimov dataset in each category of a simultaneous pdf. If None,
                     defaults to "combAsimovData_{label}" where `{label}` will be replaced by the
@@ -850,10 +917,7 @@ class ExtendedModel(object):
             if constraint_option == 0:
                 pass
             elif constraint_option == 1:
-                constrained_nuis = self.get_constrained_nuisance_parameters()
-                for nuis in constrained_nuis:
-                    nuis.setVal(0)
-                    nuis.setConstant(1)
+                self.set_constrained_nuisance_parameters_to_nominal()
             else:
                 raise ValueError(f"unsupported constraint option: {constraint_option}")
             if isinstance(nll_options, dict):
@@ -877,10 +941,18 @@ class ExtendedModel(object):
         poi_val = poi.getVal()
         
         # match values of global observables to the corresponding NPs
+        
+        if modify_globs:
+            self.match_globs()
+        
+        
+        """
         if modify_globs:
             constrained_nuis, constrained_globs = self.pair_nuis_and_glob_obs()
             for np, glob in zip(constrained_nuis, constrained_globs):
                 glob.setVal(np.getVal())
+                _PRINT_.debug("set glob {} to val {}".format(glob.GetName(), glob.getVal()))
+        """
 
         if do_fit:
             ws.saveSnapshot(con_glob_name.format(mu=poi_profile), all_globs)
@@ -916,6 +988,10 @@ class ExtendedModel(object):
         if do_import:
             getattr(ws, "import")(asimov_data)
             _PRINT_.info(f'INFO: Generated Asimov Dataset `{asimov_data_name}`')
+            
+        if asimov_snapshot is not None:
+            _PRINT_.info('INFO: Saving snapshot "{}"'.format(asimov_snapshot))
+            ws.saveSnapshot(asimov_snapshot, self.workspace.allVars())
         
         # load back a snapshot of all varaible's initial states
         ws.loadSnapshot(nom_vars_name)
