@@ -2,7 +2,7 @@ import os
 import time
 import json
 from itertools import repeat
-from typing import List, Optional
+from typing import Optional, Dict, List, Union
 
 import numpy as np
 import ROOT
@@ -81,6 +81,7 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
     if minos:
         minimize_options['minos']     = True
         minimize_options['minos_set'] = ROOT.RooArgSet(poi)
+
     # perform the fit
     minimizer.minimize(nll_commands=nll_commands, **minimize_options)
     nll = minimizer.fit_result.minNll()
@@ -158,4 +159,62 @@ def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:s
     with open(final_outname, 'w') as outfile:
         json.dump(data, outfile, indent=3)
     end_time = time.time()
-    print('INFO: All jobs have finished. Total Time taken: {:.3f} s'.format(end_time-start_time))
\ No newline at end of file
+    print('INFO: All jobs have finished. Total Time taken: {:.3f} s'.format(end_time-start_time))
+    
+
+from quickstats.components import AnalysisObject
+from quickstats.utils.common_utils import parse_config
+
+class Likelihood(AnalysisObject):
+    
+    @property
+    def fit_result(self):
+        return self.minimizer.fit_result
+    
+    @property
+    def minNll(self):
+        if not self.minimizer.fit_result:
+            return None
+        return self.minimizer.fit_result.minNll()
+    
+    def __init__(self, filename:str, data_name:str='combData', 
+                 config:Optional[Union[Dict, str]]=None):
+        
+        config = parse_config(config)
+        config['filename']  = filename
+        config['data_name'] = data_name
+        super().__init__(**config)
+        
+        self.poi_bestfit = None
+        
+    def evaluate(self, poi_val:float, poi_name:Optional[str]=None, vmin:float=10., vmax:float=10., unconditional=False,
+                 hesse:bool=False, minos:bool=True):
+        
+        self.model.workspace.saveSnapshot("tmp", self.model.workspace.allVars())
+        
+        # setup the POI
+        poi = self.model.get_poi(poi_name)
+        poi_min, poi_max = poi_val - abs(vmin), poi_val + abs(vmax)
+        poi.setRange(poi_min, poi_max)
+        
+        if unconditional:
+            poi.setConstant(0)
+            hesse = True
+        else:
+            poi.setVal(poi_val)
+            poi.setConstant(1)
+
+        minimizer_options = { 
+            'hesse': hesse,
+            'minos': minos
+        }
+        if minos:
+            minimizer_options['minos_set'] = ROOT.RooArgSet(poi)
+        
+        # perform the fit
+        self.minimizer.minimize(**minimizer_options)
+        
+        self.poi_bestfit = poi.getVal()
+        
+        self.model.workspace.loadSnapshot("tmp")
+        return self.minNll
\ No newline at end of file
