@@ -22,8 +22,8 @@ def load_macro(name='RooTwoSidedCBShape'):
     if not os.path.exists(macros_path + '.so'):
         return compile_macro(name)
     result = ROOT.gSystem.Load(macros_path)
-    if result != 0:
-        raise RuntimeError("ERROR: Shared library for the macro {} is incompatible with the current pyROOT version. "
+    if (result != 0) and (result != 1):
+        raise RuntimeError("Shared library for the macro {} is incompatible with the current pyROOT version. "
                            "Please recompile by typing \"quickstats compile\".".format(name))
     return result
 
